//
// tournament - creation of playing schedules for tournaments
// Copyright (C) 2006,2009,2010  Joachim Reichel <joachim.reichel@gmx.de>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#include "timer.h"



#include <cstddef> // NULL macro



Timer::Timer()
  : total_real_time( 0),
    total_user_time( 0),
    total_system_time( 0),
    running( false)
{
}


void Timer::start()
{
    if( !running) {
#if defined( HAVE_SYS_TIME_H)
        gettimeofday( &start_real_time, NULL);
#if defined( HAVE_SYS_RESOURCE_H)
        rusage now;
        getrusage( RUSAGE_SELF, &now);
        start_user_time = now.ru_utime;
        start_system_time = now.ru_stime;
#else
        start_user_time = start_real_time;
        start_system_time = start_real_time;
#endif // HAVE_SYS_RESOURCE_H
#else
        double now = static_cast<double>( time(NULL));
        start_real_time = now;
        start_user_time = now;
        start_system_time = now;
#endif // HAVE_SYS_TIME_H
        running = true;
    }
}


void Timer::stop()
{
    if( running) {
#if defined( HAVE_SYS_TIME_H)
        timeval now1;
        gettimeofday( &now1, NULL);
        total_real_time += interval( start_real_time, now1);
#if defined( HAVE_SYS_RESOURCE_H)
        rusage now2;
        getrusage( RUSAGE_SELF, &now2);
        total_user_time += interval( start_user_time, now2.ru_utime);
        total_system_time += interval( start_system_time, now2.ru_stime);
#else
        total_user_time += interval( start_user_time, now1);
        total_system_time += interval( start_system_time, now1);
#endif // HAVE_SYS_RESOURCE_H
#else
        double now = static_cast<double>( time(NULL));
        total_real_time += now - start_real_time;
        total_user_time += now - start_user_time;
        total_system_time += now - start_system_time;
#endif // HAVE_SYS_TIME_H
        running = false;
    }
}


void Timer::reset()
{
    running = false;
    total_real_time = 0;
    total_user_time = 0;
    total_system_time = 0;
}


void Timer::restart()
{
    total_real_time = 0;
    total_user_time = 0;
    total_system_time = 0;
#if defined( HAVE_SYS_TIME_H)
    gettimeofday( &start_real_time, NULL);
#if defined( HAVE_SYS_RESOURCE_H)
    rusage now;
    getrusage( RUSAGE_SELF, &now);
    start_user_time = now.ru_utime;
    start_system_time = now.ru_stime;
#else
    start_user_time = start_real_time;
    start_system_time = start_real_time;
#endif // HAVE_SYS_RESOURCE_H
#else
    double now = static_cast<double>( time(NULL));
    start_real_time = now;
    start_user_time = now;
    start_system_time = now;
#endif // HAVE_SYS_TIME_H
    running = true;
}


double Timer::get_real_time()
{
    if( running) {
#if defined( HAVE_SYS_TIME_H)
        timeval now;
        gettimeofday( &now, NULL);
        return total_real_time + interval( start_real_time, now);
#else
        // prevent wrap-around (begin)
        double now = static_cast<double>( time(NULL));
        total_real_time += now - start_real_time;
        total_user_time += now - start_user_time;
        total_system_time += now - start_system_time;
        start_real_time = now;
        start_user_time = now;
        start_system_time = now;
        // prevent wrap-around (end)
        return total_real_time;
#endif // HAVE_SYS_TIME_H
    } else
        return total_real_time;
}


double Timer::get_user_time()
{
    if( running) {
#if defined( HAVE_SYS_TIME_H)
#if defined( HAVE_SYS_RESOURCE_H)
        rusage now;
        getrusage( RUSAGE_SELF, &now);
        return total_user_time + interval( start_user_time, now.ru_utime);
#else
        timeval now;
        gettimeofday( &now, NULL);
        return total_user_time + interval( start_user_time, now);
#endif // HAVE_SYS_RESOURCE_H
#else
        // prevent wrap-around (begin)
        double now = static_cast<double>( time(NULL));
        total_real_time += now - start_real_time;
        total_user_time += now - start_user_time;
        total_system_time += now - start_system_time;
        start_real_time = now;
        start_user_time = now;
        start_system_time = now;
        // prevent wrap-around (end)
        return total_user_time;
#endif // HAVE_SYS_TIME_H
    } else
        return total_user_time;
}


double Timer::get_system_time()
{
    if( running) {
#if defined( HAVE_SYS_TIME_H)
#if defined( HAVE_SYS_RESOURCE_H)
        rusage now;
        getrusage( RUSAGE_SELF, &now);
        return total_system_time + interval( start_system_time, now.ru_stime);
#else
        return 0;
#endif // HAVE_SYS_RESOURCE_H
#else
        // prevent wrap-around (begin)
        double now = static_cast<double>( time(NULL));
        total_real_time += now - start_real_time;
        total_user_time += now - start_user_time;
        total_system_time += now - start_system_time;
        start_real_time = now;
        start_user_time = now;
        start_system_time = now;
        // prevent wrap-around (end)
        return 0;
#endif // HAVE_SYS_TIME_H
    } else
        return total_system_time;
}


#if defined( HAVE_SYS_TIME_H)

double Timer::interval( const timeval& t1, const timeval& t2)
{
    return 1.0 *( (t2).tv_sec -( t1).tv_sec)
           + 0.000001 *( (t2).tv_usec - (t1).tv_usec);
}



#endif // HAVE_SYS_TIME_H
